@@ -4,17 +4,20 @@ use strict;
 
 use utf8;
 use Net::Whois::Raw;
+use Data::Dumper;
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 our @EXPORT = qw( parse_whois );
 
 our $DEBUG = 0; 
 
+# parsers for parse whois text to data structure
 our %PARSERS = ( 
     'DEFAULT' => \&_default_parser,
 );
 
+# rules to convert diferent names of same fields to standard name
 our %FIELD_NAME_CONV = (
 
     # nameservers
@@ -23,6 +26,8 @@ our %FIELD_NAME_CONV = (
     name_serever  => 'nameservers',
     name_server   => 'nameservers',
     nameserver    => 'nameservers',
+    dns1          => 'nameservers',
+    dns2          => 'nameservers',
 
     # domain
     domain_name   => 'domain',
@@ -43,6 +48,16 @@ our %FIELD_NAME_CONV = (
 
 );
 
+# You can turn this flag to get
+# all values of field in all whois answers
+our $GET_ALL_VALUES = 0;
+
+# hooks for formating values
+our %HOOKS = (
+    nameservers => [ \&format_nameservers ],
+    emails => [ sub {my $value = shift; ref $value ? $value : [$value] } ],
+);
+
 # From Net::Whois::Raw
 sub import {
     my $mypkg = shift;
@@ -93,10 +108,8 @@ sub parse_whois {
 
     }
     elsif ( $args{domain} ) {
-        
         my $whois = _fetch_whois(%args);
         return $whois ? _process_parse($whois) : undef;
-
     }
     
     undef;
@@ -105,81 +118,78 @@ sub parse_whois {
 sub _process_parse {
     my ( $whois ) = @_;
 
-    my %data = ();
+    my @data = ();
     for my $ans ( @$whois ) {
 
         my $parser = 
             $ans->{srv} && $PARSERS{$ans->{srv}} ? 
                 $PARSERS{$ans->{srv}} : $PARSERS{DEFAULT};
 
-        %data = (
-            %data,
-            %{ _post_parse( $parser->($ans->{text}) ) }
-        );
+        push @data, $parser->($ans->{text});
     }
-    \%data;
+
+    _post_parse(\@data);
 }
 
-# стандартизация полученных от парсера данных
+# standardize data structure
 sub _post_parse {
     my ( $data )  = @_;
+
+    my %res = ();
+    my $count = 0;
+    my %flag = ();
+
+    for my $hash ( @$data ) {
+
+        $count++;
     
-    for my $key ( keys %$data ) {
+        for my $key ( keys %$hash ) {
+            next unless $hash->{$key};
 
-        my $value = $data->{$key};
-        
-        delete $data->{$key}, next unless $value;
-
-        # переделываем ключи
-        my $tmp_key = lc $key;
-        $tmp_key =~ s/\s+|\t+|-/_/g;
-            
-        unless ( $tmp_key eq $key ) {
-            $data->{$tmp_key} = $data->{$key};
-            delete $data->{$key};
-            $key = $tmp_key;
-        }            
-       
-        # Изменение ключа
-        if ( exists $FIELD_NAME_CONV{$key} ) {
-            delete $data->{$key};
-
-            $key =  $FIELD_NAME_CONV{$key};
-        
-            $value = [ @$value, @{$data->{$key}} ] 
-                if $data->{$key};
-
-        }
-        
-        @$value = _make_unique(@$value) if $#$value > 0;
-
-        # форматирование полей и запись в хеш
-        if ( $key eq 'nameservers' ) {
-            my @nss;
-            for my $ns ( @$value ) {
-                my ( $domain, $ip ) = split /\s+/, $ns;
-
-                $domain ||= $ns;
-                $domain =~ s/\.$//;
-                $domain = lc $domain;
-
-                push @nss, { 
-                    domain => $domain, 
-                    ( $ip ? (ip => $ip) : () )
-                }; 
+            # change keys to standard names
+            my $new_key = lc $key;
+            $new_key =~ s/\s+|\t+|-/_/g;
+            if ( exists $FIELD_NAME_CONV{$new_key} ) {
+                $new_key =  $FIELD_NAME_CONV{$new_key};
             }
-            $data->{$key} = \@nss;
+    
+            unless ( $GET_ALL_VALUES ) {
+                if ( exists $res{$new_key} && !$flag{$new_key} ) {
+                    delete $res{$new_key};
+                    $flag{$new_key} = 1;
+                }
+            }
+
+            # add values to result hash           
+            if ( exists $res{$new_key} ) { 
+                push @{$res{$new_key}}, @{$hash->{$key}};
+            }
+            else {
+                $res{$new_key} = ref $hash->{$key} ? $hash->{$key} : [$hash->{$key}];
+            }
+        
         }
-        elsif ( $key eq 'emails' ) {
-            $data->{$key} = $value;
+    }
+
+    # make unique and process hooks
+    while ( my ( $key, $value ) = each %res ) {   
+ 
+        if ( scalar @$value > 1 ) {
+            @$value = _make_unique(@$value);
         }
         else {
-            $data->{$key} = scalar @$value > 1 ? $value : $value->[0];
-            
-        } 
-        
+            $value = $value->[0];
+        }
+
+        if ( exists $HOOKS{$key} ) {
+            for my $hook ( @{$HOOKS{$key}} ) { $value = $hook->($value) }
+        }
+    
+        $res{$key} = $value;
+
     }
-    $data;
+
+    \%res;
 }
 
 sub _make_unique {
@@ -297,7 +307,7 @@ sub _default_parser {
     my ( $raw ) = @_;
     my %data;    
     
-    # получаем данные в виде ключ => значение
+    # transform data to key => value
     for my $line ( split /\n/, $raw ) {
 
         chomp $line;
@@ -309,21 +319,44 @@ sub _default_parser {
         $key =~ s/\s+$//;
         $value =~ s/\s+$//;
 
-        # если полей с одинаковым ключем несколько, пихаем в массив
+        # if we have more then one value for one field we push them into array
         $data{$key} = ref $data{$key} eq 'ARRAY' ? 
             [ @{$data{$key}}, $value ] : [ $value ];
 
     }
 
-    # поиск вообще всех emails
+    # find all emails in the text
     my @emails = $raw =~ /($RFC822PAT)/gso;
     @emails = map { $_ =~ s/\s+//g; ($_) } @emails;
     $data{emails} = exists $data{emails} ? 
-        [ @{$data{emails}}, @emails ] : [@emails];
+        [ @{$data{emails}}, @emails ] : \@emails;
    
     \%data;
 }
 
+## FORMATERS ##
+
+sub format_nameservers {
+    my ( $value ) = @_;
+    
+    $value = [$value] unless ref $value;
+
+    my @nss;
+    for my $ns ( @$value ) {
+        my ( $domain, $ip ) = split /\s+/, $ns;
+
+        $domain ||= $ns;
+        $domain =~ s/\.$//;
+        $domain = lc $domain;
+
+        push @nss, { 
+            domain => $domain, 
+            ( $ip ? (ip => $ip) : () )
+        }; 
+    }
+
+    \@nss;
+}
 
 1;
 
@@ -368,11 +401,24 @@ Net::Whois::Parser - module for parsing whois information
     
     $Net::Whois::Parser::PARSERS{'whois.example.com'} = \&my_parser;
     $Net::Whois::Parser::PARSERS{'DEFAULT'}           = \&my_default_parser;
-    
+
+    # If you want to get all values of fields from all whois answers
+    $Net::Whois::Parser::GET_ALL_VALUES = 1;
+        # example
+        # Net::Whois::Raw returns 2 answers
+        $raw = [ { text => 'key: value1' }, { text => 'key: value2'}];
+        $data = parse_whois(raw => $raw);
+        # If flag is off parser returns
+        # { key => 'value2' };
+        # If flag is on parser returns
+        # { key => [ 'value1', 'value2' ] };
     
     # If you want to convert some field name to another:
-        
     $Net::Whois::Parser::FIELD_NAME_CONV{'Domain name'} = 'domain';
+
+    # If you want to format some fields.
+    # I think it is very usefull for dates.
+    $Net::Whois::Parser::HOOKS{'expiration_date'} = [ \&format_date ];
     
 =head1 DESCRIPTION
 
